@@ -2,7 +2,7 @@
 
 /* otf.{cc,hh} -- OpenType font basics
  *
- * Copyright (c) 2002-2004 Eddie Kohler
+ * Copyright (c) 2002-2006 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <algorithm>
 #include <efont/otfdata.hh>	// for ntohl()
+#include <efont/otfname.hh>
 
 #ifndef static_assert
 #define static_assert(c) switch (c) case 0: case (c):
@@ -41,7 +42,7 @@ Font::Font(const String &s, ErrorHandler *errh)
     : _str(s)
 {
     _str.align(4);
-    _error = parse_header(errh ? errh : ErrorHandler::ignore_handler());
+    _error = parse_header(errh ? errh : ErrorHandler::silent_handler());
 }
 
 int
@@ -83,10 +84,42 @@ Font::parse_header(ErrorHandler *errh)
 	    return errh->error("OTF data for '%s' out of range", Tag(tag).text().c_str()), -EFAULT;
 	last_tag = tag;
     }
-    
+
     return 0;
 }
 
+bool
+Font::check_checksums(ErrorHandler *errh) const
+{
+    if (error() < 0)
+	return false;
+    int nt = ntables();
+    bool ok = true;
+    for (int i = 0; i < nt; i++) {
+	const uint8_t *entry = data() + HEADER_SIZE + TABLE_DIR_ENTRY_SIZE * i;
+	String tbl = _str.substring(ULONG_AT(entry + 8), ULONG_AT(entry + 12));
+	uint32_t sum = checksum(tbl);
+	if (ULONG_AT(entry) == 0x68656164	// 'head'
+	    && tbl.length() >= 12)
+	    sum -= ULONG_AT(tbl.data() + 8);
+	if (sum != ULONG_AT(entry + 4)) {
+	    if (errh)
+		errh->error("table '%s' checksum error: %x vs. %x", Tag(ULONG_AT(entry)).text().c_str(), sum, ULONG_AT(entry + 4));
+	    ok = false;
+	}
+    }
+    return ok;
+}
+
+int
+Font::ntables() const
+{
+    if (error() < 0)
+	return 0;
+    else
+	return USHORT_AT(data() + 4);
+}
+
 String
 Font::table(Tag tag) const
 {
@@ -99,13 +132,16 @@ Font::table(Tag tag) const
 	return String();
 }
 
-int
-Font::ntables() const
+uint32_t
+Font::table_checksum(Tag tag) const
 {
     if (error() < 0)
 	return 0;
+    const uint8_t *entry = tag.table_entry(data() + HEADER_SIZE, USHORT_AT(data() + 4), TABLE_DIR_ENTRY_SIZE);
+    if (entry)
+	return ULONG_AT(entry + 4);
     else
-	return USHORT_AT(data() + 4);
+	return 0;
 }
 
 Tag
@@ -117,6 +153,111 @@ Font::table_tag(int i) const
 	return Tag(ULONG_AT(data() + HEADER_SIZE + TABLE_DIR_ENTRY_SIZE * i));
 }
 
+uint32_t
+Font::checksum(const uint8_t *begin, const uint8_t *end)
+{
+    uint32_t sum = 0;
+    if (reinterpret_cast<uintptr_t>(begin) % 4)
+	for (; begin + 3 < end; begin += 4)
+	    sum += (begin[0] << 24) + (begin[1] << 16) + (begin[2] << 8) + begin[3];
+    else
+	for (; begin + 3 < end; begin += 4)
+	    sum += ULONG_AT(begin);
+    uint32_t leftover = 0;
+    for (int i = 0; i < 4; i++)
+	leftover = (leftover << 8) + (begin < end ? *begin++ : 0);
+    return sum + leftover;
+}
+
+uint32_t
+Font::checksum(const String &s)
+{
+    return checksum(reinterpret_cast<const uint8_t *>(s.begin()), reinterpret_cast<const uint8_t *>(s.end()));
+}
+
+namespace {
+class TagCompar { public:
+    TagCompar(const Vector<Tag> &tags) : _tags(tags) { }
+    bool operator()(int a, int b) { return _tags[a] < _tags[b]; }
+    const Vector<Tag> &_tags;
+};
+}
+
+Font
+Font::make(bool truetype, const Vector<Tag>& tags, const Vector<String>& data)
+{
+    StringAccum sa;
+
+    // create offset table
+    {
+	union {
+	    uint8_t c[HEADER_SIZE];
+	    uint16_t s[HEADER_SIZE / 2];
+	    uint32_t l[HEADER_SIZE / 4];
+	} hdr;
+	hdr.l[0] = (truetype ? htonl(0x00010000) : htonl(0x4F54544F));
+	hdr.s[2] = htons(tags.size());
+	int entrySelector;
+	for (entrySelector = 0; (2 << entrySelector) <= tags.size(); entrySelector++)
+	    /* nada */;
+	hdr.s[3] = htons((1 << entrySelector) * 16);
+	hdr.s[4] = htons(entrySelector);
+	hdr.s[5] = htons((tags.size() - (1 << entrySelector)) * 16);
+	sa.append(&hdr.c[0], HEADER_SIZE);
+    }
+
+    // sort tags
+    Vector<int> permut;
+    for (int i = 0; i < tags.size(); i++)
+	permut.push_back(i);
+    std::sort(permut.begin(), permut.end(), TagCompar(tags));
+
+    // table listing
+    uint32_t offset = HEADER_SIZE + TABLE_DIR_ENTRY_SIZE * tags.size();
+    for (int *tp = permut.begin(); tp < permut.end(); tp++) {
+	union {
+	    uint8_t c[TABLE_DIR_ENTRY_SIZE];
+	    uint32_t l[TABLE_DIR_ENTRY_SIZE / 4];
+	} tdir;
+	tdir.l[0] = htonl(tags[*tp].value());
+
+	// discount current checksum adjustment in head table
+	uint32_t sum = checksum(data[*tp]);
+	if (tags[*tp] == Tag("head") && data[*tp].length() >= 12) {
+	    const uint8_t *x = data[*tp].udata() + 8;
+	    sum -= (x[0] << 24) + (x[1] << 16) + (x[2] << 8) + x[3];
+	}
+	tdir.l[1] = htonl(sum);
+
+	tdir.l[2] = htonl(offset);
+	tdir.l[3] = htonl(data[*tp].length());
+	sa.append(&tdir.c[0], TABLE_DIR_ENTRY_SIZE);
+	offset += (data[*tp].length() + 3) & ~3;
+    }
+
+    // actual tables
+    for (int *tp = permut.begin(); tp < permut.end(); tp++) {
+	sa << data[*tp];
+	while (sa.length() % 4)
+	    sa << '\0';
+    }
+
+    // fix 'head' table
+    for (int i = 0; i < tags.size(); i++) {
+	char *thdr = sa.data() + HEADER_SIZE + TABLE_DIR_ENTRY_SIZE * i;
+	if (ULONG_AT(thdr) == 0x68656164 && ULONG_AT(thdr + 12) >= 12) {
+	    uint32_t offset = ULONG_AT(thdr + 8);
+	    char *head = sa.data() + offset;
+	    memset(head + 8, '\0', 4);
+	    uint32_t allsum = checksum(reinterpret_cast<uint8_t *>(sa.data()), reinterpret_cast<uint8_t *>(sa.data() + sa.length()));
+	    uint32_t *adj = reinterpret_cast<uint32_t *>(head + 8);
+	    *adj = htonl(0xB1B0AFBA - allsum);
+	}
+    }
+
+    return Font(sa.take_string());
+}
+
 
 /**************************
  * Tag                    *
@@ -195,14 +336,13 @@ const uint8_t *
 Tag::table_entry(const uint8_t *table, int n, int entry_size) const
 {
     assert(((uintptr_t)table & 1) == 0);
-    int l = 0;
-    int r = n - 1;
-    while (l <= r) {
-	int m = (l + r) / 2;
+    int l = 0, r = n;
+    while (l < r) {
+	int m = l + (r - l) / 2;
 	const uint8_t *entry = table + m * entry_size;
 	uint32_t m_tag = ULONG_AT2(entry);
 	if (_tag < m_tag)
-	    r = m - 1;
+	    r = m;
 	else if (_tag == m_tag)
 	    return entry;
 	else
@@ -223,7 +363,7 @@ ScriptList::assign(const String &str, ErrorHandler *errh)
 {
     _str = str;
     _str.align(4);
-    int result = check_header(errh ? errh : ErrorHandler::ignore_handler());
+    int result = check_header(errh ? errh : ErrorHandler::silent_handler());
     if (result < 0)
 	_str = String();
     return result;
@@ -305,7 +445,7 @@ ScriptList::language_systems(Vector<Tag> &script, Vector<Tag> &langsys, ErrorHan
 {
     script.clear();
     langsys.clear();
-    
+
     const uint8_t *data = _str.udata();
     int nscripts = USHORT_AT(data);
     for (int i = 0; i < nscripts; i++) {
@@ -322,7 +462,7 @@ ScriptList::language_systems(Vector<Tag> &script, Vector<Tag> &langsys, ErrorHan
 	    script.push_back(script_tag), langsys.push_back(langsys_tag);
 	}
     }
-    
+
     return 0;
 }
 
@@ -332,7 +472,7 @@ ScriptList::features(Tag script, Tag langsys, int &required_fid, Vector<int> &fi
     required_fid = -1;
     if (clear_fids)
 	fids.clear();
-    
+
     int offset = langsys_offset(script, langsys);
     if (offset <= 0)
 	return offset;
@@ -352,7 +492,7 @@ ScriptList::features(Tag script, Tag langsys, int &required_fid, Vector<int> &fi
     data += offset + 6;
     for (int i = 0; i < featureCount; i++, data += FEATURE_RECSIZE)
 	fids.push_back(USHORT_AT(data));
-    
+
     return 0;
 }
 
@@ -368,7 +508,7 @@ FeatureList::assign(const String &str, ErrorHandler *errh)
 {
     _str = str;
     _str.align(2);
-    int result = check_header(errh ? errh : ErrorHandler::ignore_handler());
+    int result = check_header(errh ? errh : ErrorHandler::silent_handler());
     if (result < 0)
 	_str = String();
     return result;
@@ -404,7 +544,7 @@ FeatureList::params(int fid, int length, ErrorHandler *errh, bool old_style_offs
     if (_str.length() == 0 || length < 0)
 	return String();
     if (errh == 0)
-	errh = ErrorHandler::ignore_handler();
+	errh = ErrorHandler::silent_handler();
 
     const uint8_t *data = _str.udata();
     int len = _str.length();
@@ -425,6 +565,44 @@ FeatureList::params(int fid, int length, ErrorHandler *errh, bool old_style_offs
 	return _str.substring(poff, length);
 }
 
+String
+FeatureList::size_params(int fid, const Name &name, ErrorHandler *errh) const
+{
+    // implement 'size' checks from Read Roberts
+    for (int i = 0; i < 2; i++) {
+	String s = params(fid, 10, errh, i != 0);
+	const uint8_t *data = s.udata();
+	// errh->message("trying %d %d %d %d %d\n", USHORT_AT(data), USHORT_AT(data + 2), USHORT_AT(data + 4), USHORT_AT(data + 6), USHORT_AT(data + 8));
+	if (USHORT_AT(data) == 0)		// design size == 0
+	    continue;
+	if (USHORT_AT(data + 2) == 0		// subfamily ID == 0
+	    && USHORT_AT(data + 6) == 0		// range start == 0
+	    && USHORT_AT(data + 8) == 0		// range end == 0
+	    && USHORT_AT(data + 4) == 0)	// menu name ID == 0
+	    return s;
+	if (USHORT_AT(data + 6) >= USHORT_AT(data + 8) // range start >= range end
+	    || USHORT_AT(data + 4) < 256	// menu name ID < 256
+	    || USHORT_AT(data + 4) > 32767	// menu name ID > 32767
+	    || !name.english_name(USHORT_AT(data + 4))) // menu name ID is a name ID defined by the font
+	    continue;
+	if (USHORT_AT(data) + 1 >= USHORT_AT(data + 6) // design size >= range start (with 1 dp grace)
+	    && USHORT_AT(data) <= USHORT_AT(data + 8) + 1) // design size <= range end (with 1 dp grace)
+	    return s;
+	else if (i == 1				// old-style feature
+		 && USHORT_AT(data + 8) <= 1440	// range end <= 144 point
+		 && USHORT_AT(data) <= 1440) {	// design size <= 144 point
+	    // some old fonts define a bogus feature with design size
+	    // not in range (John Owens, Read Roberts)
+	    if (errh)
+		errh->warning("invalid 'size' feature: design size not in range");
+	    return s;
+	}
+    }
+    if (errh)
+	errh->error("no valid 'size' feature data in the 'size' feature");
+    return String();
+}
+
 int
 FeatureList::find(Tag tag, const Vector<int> &fids) const
 {
@@ -450,13 +628,13 @@ FeatureList::filter(Vector<int> &fids, const Vector<Tag> &sorted_ftags) const
 	fids.clear();
     else {
 	std::sort(fids.begin(), fids.end()); // sort fids
-	
+
 	int i = 0, j = 0;
 	while (i < fids.size() && fids[i] < 0)
 	    fids[i++] = 0x7FFFFFFF;
 
 	// XXX check that feature list is in alphabetical order
-	
+
 	const uint8_t *data = _str.udata();
 	int nfeatures = USHORT_AT(data);
 	while (i < fids.size() && j < sorted_ftags.size() && fids[i] < nfeatures) {
@@ -486,7 +664,7 @@ FeatureList::lookups(int fid, Vector<int> &results, ErrorHandler *errh, bool cle
     if (_str.length() == 0)
 	return -1;
     if (errh == 0)
-	errh = ErrorHandler::ignore_handler();
+	errh = ErrorHandler::silent_handler();
 
     const uint8_t *data = _str.udata();
     int len = _str.length();
@@ -587,7 +765,7 @@ Coverage::Coverage(const String &str, ErrorHandler *errh, bool do_check) throw (
 {
     _str.align(2);
     if (do_check) {
-	if (check(errh ? errh : ErrorHandler::ignore_handler()) < 0)
+	if (check(errh ? errh : ErrorHandler::silent_handler()) < 0)
 	    _str = String();
     } else {			// check()'s shorten-string side effect
 	const uint8_t *data = _str.udata();
@@ -655,17 +833,17 @@ Coverage::coverage_index(Glyph g) const throw ()
 {
     if (_str.length() == 0)
 	return -1;
-    
+
     const uint8_t *data = _str.udata();
     int count = USHORT_AT(data + 2);
     if (data[1] == T_LIST) {
-	int l = 0, r = count - 1;
+	int l = 0, r = count;
 	data += HEADERSIZE;
-	while (l <= r) {
-	    int m = (l + r) >> 1;
+	while (l < r) {
+	    int m = l + (r - l) / 2;
 	    int mval = USHORT_AT(data + m * LIST_RECSIZE);
 	    if (g < mval)
-		r = m - 1;
+		r = m;
 	    else if (g == mval)
 		return m;
 	    else
@@ -673,13 +851,13 @@ Coverage::coverage_index(Glyph g) const throw ()
 	}
 	return -1;
     } else if (data[1] == T_RANGES) {
-	int l = 0, r = count - 1;
+	int l = 0, r = count;
 	data += HEADERSIZE;
-	while (l <= r) {
-	    int m = (l + r) >> 1;
+	while (l < r) {
+	    int m = l + (r - l) / 2;
 	    const uint8_t *rec = data + m * RANGES_RECSIZE;
 	    if (g < USHORT_AT(rec))
-		r = m - 1;
+		r = m;
 	    else if (g <= USHORT_AT(rec + 2))
 		return USHORT_AT(rec + 4) + g - USHORT_AT(rec);
 	    else
@@ -695,20 +873,20 @@ Coverage::operator[](int cindex) const throw ()
 {
     if (_str.length() == 0 || cindex < 0)
 	return 0;
-    
+
     const uint8_t *data = _str.udata();
     int count = USHORT_AT(data + 2);
     if (data[1] == T_LIST)
 	return (cindex < count ? USHORT_AT(data + cindex * LIST_RECSIZE) : 0);
     else if (data[1] == T_RANGES) {
-	int l = 0, r = count - 1;
+	int l = 0, r = count;
 	data += HEADERSIZE;
-	while (l <= r) {
-	    int m = (l + r) >> 1;
+	while (l < r) {
+	    int m = l + (r - l) / 2;
 	    const uint8_t *rec = data + m * RANGES_RECSIZE;
 	    int start_cindex = USHORT_AT(rec + 4);
 	    if (cindex < start_cindex)
-		r = m - 1;
+		r = m;
 	    else if (cindex < start_cindex + USHORT_AT(rec + 2) - USHORT_AT(rec))
 		return USHORT_AT(rec) + cindex - start_cindex;
 	    else
@@ -847,7 +1025,7 @@ bool
 Coverage::iterator::forward_to(Glyph find)
 {
     // XXX really should check that this works
-    
+
     if (find <= _value)
 	return find == _value;
     else if (_pos >= _str.length())
@@ -863,18 +1041,18 @@ Coverage::iterator::forward_to(Glyph find)
 	    _value = USHORT_AT(data + _pos);
 	    return find == _value;
 	}
-	
+
 	// otherwise, binary search over remaining area
 	int l = ((_pos - HEADERSIZE) / LIST_RECSIZE) + 1;
-	int r = (_str.length() - HEADERSIZE) / LIST_RECSIZE - 1;
+	int r = (_str.length() - HEADERSIZE) / LIST_RECSIZE;
 	data += HEADERSIZE;
-	while (l <= r) {
-	    int m = (l + r) >> 1;
+	while (l < r) {
+	    int m = l + (r - l) / 2;
 	    Glyph g = USHORT_AT(data + m * LIST_RECSIZE);
 	    if (find < g)
-		r = m - 1;
+		r = m;
 	    else if (find == g)
-		l = m, r = m - 1;
+		l = r = m;
 	    else
 		l = m + 1;
 	}
@@ -897,12 +1075,12 @@ Coverage::iterator::forward_to(Glyph find)
 
 	// otherwise, binary search over remaining area
 	int l = ((_pos - HEADERSIZE) / RANGES_RECSIZE) + 1;
-	int r = (_str.length() - HEADERSIZE) / RANGES_RECSIZE - 1;
+	int r = (_str.length() - HEADERSIZE) / RANGES_RECSIZE;
 	data += HEADERSIZE;
-	while (l <= r) {
-	    int m = (l + r) >> 1;
+	while (l < r) {
+	    int m = l + (r - l) / 2;
 	    if (find < USHORT_AT(data + m * RANGES_RECSIZE))
-		r = m - 1;
+		r = m;
 	    else if (find <= USHORT_AT(data + m * RANGES_RECSIZE + 2)) {
 		_pos = HEADERSIZE + m * RANGES_RECSIZE;
 		_value = find;
@@ -987,7 +1165,7 @@ ClassDef::ClassDef(const String &str, ErrorHandler *errh) throw ()
     : _str(str)
 {
     _str.align(2);
-    if (check(errh ? errh : ErrorHandler::ignore_handler()) < 0)
+    if (check(errh ? errh : ErrorHandler::silent_handler()) < 0)
 	_str = String();
 }
 
@@ -1027,10 +1205,10 @@ ClassDef::lookup(Glyph g) const throw ()
 {
     if (_str.length() == 0)
 	return -1;
-    
+
     const uint8_t *data = _str.udata();
     int coverageFormat = USHORT_AT(data);
-    
+
     if (coverageFormat == T_LIST) {
 	Glyph start = USHORT_AT(data + 2);
 	int count = USHORT_AT(data + 4);
@@ -1039,13 +1217,13 @@ ClassDef::lookup(Glyph g) const throw ()
 	else
 	    return USHORT_AT(data + LIST_HEADERSIZE + (g - start) * LIST_RECSIZE);
     } else if (coverageFormat == T_RANGES) {
-	int l = 0, r = USHORT_AT(data + 2) - 1;
+	int l = 0, r = USHORT_AT(data + 2);
 	data += RANGES_HEADERSIZE;
-	while (l <= r) {
-	    int m = (l + r) >> 1;
+	while (l < r) {
+	    int m = l + (r - l) / 2;
 	    const uint8_t *rec = data + m * RANGES_RECSIZE;
 	    if (g < USHORT_AT(rec))
-		r = m - 1;
+		r = m;
 	    else if (g <= USHORT_AT(rec + 2))
 		return USHORT_AT(rec + 4);
 	    else
@@ -1152,7 +1330,7 @@ ClassDef::class_iterator::increment_class0()
     const uint8_t *data = _str.udata();
     int len = _str.length();
     bool is_list = (data[1] == T_LIST);
-    
+
     if (_pos != 0)
 	_coviter++;
     else
@@ -1198,7 +1376,7 @@ ClassDef::class_iterator::operator++(int)
 	increment_class0();
 	return;
     }
-    
+
     const uint8_t *data = _str.udata();
     int len = _str.length();
     bool is_list = (data[1] == T_LIST);
