@@ -1,6 +1,6 @@
 /* t1minimize.cc -- make minimal copy of a Type 1 font
  *
- * Copyright (c) 2003-2004 Eddie Kohler
+ * Copyright (c) 2003-2006 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -20,20 +20,18 @@
 
 using namespace Efont;
 
-/*****
- * main
- **/
-
 Type1Font *
 minimize(Type1Font *font)
 {
-    Type1Font *output = Type1Font::skeleton_make_copy(font, font->font_name());
+    Vector<double> xuid_extension;
+    xuid_extension.push_back(0x000395C1);
+    Type1Font *output = Type1Font::skeleton_make_copy(font, font->font_name(), &xuid_extension);
 
     // Subrs
     for (int i = 0; i < font->nsubrs(); i++)
 	if (Type1Subr *s = font->subr_x(i))
 	    output->set_subr(s->subrno(), s->t1cs(), s->definer());
-    
+
     // CharStrings
     for (int i = 0; i < font->nglyphs(); i++)
 	if (Type1Subr *g = font->glyph_x(i))
