@@ -1,6 +1,6 @@
 /* ttftotype42.cc -- driver for translating TrueType fonts to Type 42 fonts
  *
- * Copyright (c) 2006 Eddie Kohler
+ * Copyright (c) 2006-2009 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -48,9 +48,9 @@ using namespace Efont;
 #define QUIET_OPT	303
 #define OUTPUT_OPT	306
 
-Clp_Option options[] = {
+const Clp_Option options[] = {
     { "help", 'h', HELP_OPT, 0, 0 },
-    { "output", 'o', OUTPUT_OPT, Clp_ArgString, 0 },
+    { "output", 'o', OUTPUT_OPT, Clp_ValString, 0 },
     { "quiet", 'q', QUIET_OPT, 0, Clp_Negate },
     { "version", 'v', VERSION_OPT, 0, 0 },
 };
@@ -67,7 +67,7 @@ usage_error(ErrorHandler *errh, const char *error_message, ...)
     if (!error_message)
 	errh->message("Usage: %s [OPTIONS] [FONTFILE [OUTPUTFILE]]", program_name);
     else
-	errh->verror(ErrorHandler::ERR_ERROR, String(), error_message, val);
+	errh->vxmessage(ErrorHandler::e_error, error_message, val);
     errh->message("Type %s --help for more information.", program_name);
     exit(1);
 }
@@ -75,8 +75,9 @@ usage_error(ErrorHandler *errh, const char *error_message, ...)
 void
 usage()
 {
-    printf("\
-'Ttftotype42' translates a TrueType or TrueType-flavored OpenType font into\n\
+    FileErrorHandler uerrh(stdout);
+    uerrh.message("\
+%<Ttftotype42%> translates a TrueType or TrueType-flavored OpenType font into\n\
 PostScript Type 42 format, which is suitable for inclusion in PostScript\n\
 files. The result, is usually written to the standard output.\n\
 \n\
@@ -88,7 +89,7 @@ Options:\n\
   -h, --help                   Print this message and exit.\n\
   -v, --version                Print version number and exit.\n\
 \n\
-Report bugs to <kohler@cs.ucla.edu>.\n", program_name);
+Report bugs to <ekohler@gmail.com>.\n", program_name);
 }
 
 
@@ -162,7 +163,7 @@ do_file(const char *infn, const char *outfn, ErrorHandler *errh)
 #endif
     } else if (!(f = fopen(infn, "rb")))
 	errh->fatal("%s: %s", infn, strerror(errno));
-  
+
     int c = getc(f);
     ungetc(c, f);
 
@@ -198,7 +199,7 @@ do_file(const char *infn, const char *outfn, ErrorHandler *errh)
 	    tables.push_back(s);
 	}
     OpenType::Font reduced_font = OpenType::Font::make(true, tags, tables);
-    
+
     // output file
     if (!outfn || strcmp(outfn, "-") == 0) {
 	f = stdout;
@@ -212,7 +213,7 @@ do_file(const char *infn, const char *outfn, ErrorHandler *errh)
 #endif
 
     // fprintf(f, "%%!\n");
-    
+
     // get glyph names
     TrueTypeBoundsCharstringProgram ttbprog(&otf);
     Vector<PermString> gn;
@@ -271,7 +272,7 @@ do_file(const char *infn, const char *outfn, ErrorHandler *errh)
 	fprintf(f, "/UnderlineThickness %g def\n", post.underline_thickness() / emunits);
     }
     fprintf(f, "end readonly def\n");
-    
+
     // encoding
     fprintf(f, "/Encoding 256 array\n0 1 255{1 index exch/.notdef put}for\n");
     for (int i = 0; i < 256; i++)
@@ -301,7 +302,7 @@ do_file(const char *infn, const char *outfn, ErrorHandler *errh)
     fprintf(f, "FontName currentdict end definefont pop\n");
 
     // fprintf(f, "/%s 100 selectfont 30 30 moveto (Hello! 9) show showpage\n", name.english_name(OpenType::Name::N_POSTSCRIPT).c_str());
-    
+
     if (f != stdout)
 	fclose(f);
 }
@@ -312,11 +313,11 @@ main(int argc, char *argv[])
     Clp_Parser *clp =
 	Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
     program_name = Clp_ProgramName(clp);
-  
+
     ErrorHandler *errh = ErrorHandler::static_initialize(new FileErrorHandler(stderr, String(program_name) + ": "));
     const char *input_file = 0;
     const char *output_file = 0;
-  
+
     while (1) {
 	int opt = Clp_Next(clp);
 	switch (opt) {
@@ -327,16 +328,16 @@ main(int argc, char *argv[])
 	    else
 		errh = new SilentErrorHandler;
 	    break;
-      
+
 	  case VERSION_OPT:
 	    printf("ttftotype42 (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2006 Eddie Kohler\n\
+	    printf("Copyright (C) 2006-2009 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
 	    exit(0);
 	    break;
-      
+
 	  case HELP_OPT:
 	    usage();
 	    exit(0);
@@ -346,7 +347,7 @@ particular purpose.\n");
 	  output_file:
 	    if (output_file)
 		usage_error(errh, "output file specified twice");
-	    output_file = clp->arg;
+	    output_file = clp->vstr;
 	    break;
 
 	  case Clp_NotOption:
@@ -355,24 +356,24 @@ particular purpose.\n");
 	    else if (input_file)
 		goto output_file;
 	    else
-		input_file = clp->arg;
+		input_file = clp->vstr;
 	    break;
-      
+
 	  case Clp_Done:
 	    goto done;
-      
+
 	  case Clp_BadOption:
 	    usage_error(errh, 0);
 	    break;
-      
+
 	  default:
 	    break;
-      
+
 	}
     }
-  
+
   done:
     do_file(input_file, output_file, errh);
-    
+
     return (errh->nerrors() == 0 ? 0 : 1);
 }
